"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const no_partial_division_1 = __importDefault(require("./no-partial-division"));
const utils_1 = require("@typescript-eslint/utils");
const utils_2 = require("@typescript-eslint/utils");
// eslint-disable-next-line functional/prefer-immutable-types
const ruleTester = new utils_2.ESLintUtils.RuleTester({
    parserOptions: {
        sourceType: "module",
        project: "./tsconfig.tests.json",
    },
    parser: "@typescript-eslint/parser",
});
// eslint-disable-next-line functional/no-expression-statements
ruleTester.run("no-partial-division", no_partial_division_1.default, {
    valid: [
        {
            filename: "file.ts",
            code: `
        1 / 1;
      `,
        },
        {
            filename: "file.ts",
            code: `
        1n / 1n;
      `,
        },
        {
            filename: "file.ts",
            code: `
        1 + 1;
      `,
        },
        {
            filename: "file.ts",
            code: `
        const foo = 1;
        const bar = 1 as const;
        const result = foo / bar;
      `,
        },
        {
            filename: "file.ts",
            code: `
        const foo = 1;
        const bar = 1;
        const result = foo / bar;
      `,
        },
        {
            filename: "file.ts",
            code: `
        const foo: bigint = 1n;
        const bar = 1n as const;
        const result = foo / bar;
      `,
        },
        {
            filename: "file.ts",
            code: `
        const foo = 1n;
        const bar = 1n;
        const result = foo / bar;
      `,
        },
        {
            filename: "file.ts",
            code: `
        declare const foo: 42 | 43;
        const bar = 1 / foo;
      `,
        },
        {
            filename: "file.ts",
            code: `
        declare const foo: 43 & { __tag: string };
        const bar = 1 / foo;
      `,
        },
        {
            filename: "file.ts",
            code: `
        declare const foo: 43n & { __tag: string };
        const bar = 1n / foo;
      `,
        },
    ],
    invalid: [
        {
            filename: "file.ts",
            code: `
        const result = 1 / 0;
      `,
            errors: [
                {
                    messageId: "errorStringGeneric",
                    type: utils_1.AST_NODE_TYPES.BinaryExpression,
                },
            ],
        },
        {
            filename: "file.ts",
            code: `
        const result = 1 / 0.0;
      `,
            errors: [
                {
                    messageId: "errorStringGeneric",
                    type: utils_1.AST_NODE_TYPES.BinaryExpression,
                },
            ],
        },
        {
            filename: "file.ts",
            code: `
        const result = 1 / -0;
      `,
            errors: [
                {
                    messageId: "errorStringGeneric",
                    type: utils_1.AST_NODE_TYPES.BinaryExpression,
                },
            ],
        },
        {
            filename: "file.ts",
            code: `
        const result = 1n / 0n;
      `,
            errors: [
                {
                    messageId: "errorStringGeneric",
                    type: utils_1.AST_NODE_TYPES.BinaryExpression,
                },
            ],
        },
        {
            filename: "file.ts",
            code: `
        const result = 1n / -0n;
      `,
            errors: [
                {
                    messageId: "errorStringGeneric",
                    type: utils_1.AST_NODE_TYPES.BinaryExpression,
                },
            ],
        },
        {
            filename: "file.ts",
            code: `
        const foo: bigint = 1n;
        const bar: bigint = 0n;
        const result = foo / bar;
      `,
            errors: [
                {
                    messageId: "errorStringGeneric",
                    type: utils_1.AST_NODE_TYPES.BinaryExpression,
                },
            ],
        },
        {
            filename: "file.ts",
            code: `
        const foo: bigint = 1n;
        const bar: bigint = 1n;
        const result = foo / bar;
      `,
            errors: [
                {
                    messageId: "errorStringGeneric",
                    type: utils_1.AST_NODE_TYPES.BinaryExpression,
                },
            ],
        },
        {
            filename: "file.ts",
            code: `
        const foo = 1n;
        const bar = 0n;
        const result = foo / bar;
      `,
            errors: [
                {
                    messageId: "errorStringGeneric",
                    type: utils_1.AST_NODE_TYPES.BinaryExpression,
                },
            ],
        },
        {
            filename: "file.ts",
            code: `
        declare const foo: 42 | 0;
        const bar = 1 / foo;
      `,
            errors: [
                {
                    messageId: "errorStringGeneric",
                    type: utils_1.AST_NODE_TYPES.BinaryExpression,
                },
            ],
        },
        {
            filename: "file.ts",
            code: `
        declare const foo: 42 | number;
        const bar = 1 / foo;
      `,
            errors: [
                {
                    messageId: "errorStringGeneric",
                    type: utils_1.AST_NODE_TYPES.BinaryExpression,
                },
            ],
        },
        {
            filename: "file.ts",
            code: `
        declare const foo: number & { __tag: string };
        const bar = 1 / foo;
      `,
            errors: [
                {
                    messageId: "errorStringGeneric",
                    type: utils_1.AST_NODE_TYPES.BinaryExpression,
                },
            ],
        },
    ],
});
