"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable functional/prefer-immutable-types */
const is_immutable_type_1 = require("is-immutable-type");
const common_1 = require("./common");
const unsafe_assignment_rule_1 = require("./unsafe-assignment-rule");
/**
 * An ESLint rule to ban unsafe assignment from mutable to readonly types.
 */
// eslint-disable-next-line total-functions/no-unsafe-readonly-mutable-assignment
const noUnsafeMutableReadonlyAssignment = (0, common_1.createRule)({
    name: "no-unsafe-mutable-readonly-assignment",
    meta: {
        type: "problem",
        docs: {
            description: "Bans unsafe assignment from mutable to readonly types.",
            recommended: "error",
        },
        messages: {
            errorStringGeneric: 'Unsafe mutable to readonly assignment. Source: "{{ source }}", destination: "{{ destination }}"',
        },
        schema: [],
    },
    create: (0, unsafe_assignment_rule_1.createNoUnsafeAssignmentRule)((checker, rawDestinationType, rawSourceType) => {
        const typePairs = (0, common_1.assignableTypePairs)(rawDestinationType, rawSourceType, checker);
        return typePairs.some(({ sourceType, destinationType }) => {
            const destinationImmutability = (0, is_immutable_type_1.getTypeImmutability)(checker, destinationType);
            const sourceImmutability = (0, is_immutable_type_1.getTypeImmutability)(checker, sourceType);
            return ((destinationImmutability === is_immutable_type_1.Immutability.Immutable ||
                destinationImmutability === is_immutable_type_1.Immutability.ReadonlyDeep) &&
                (sourceImmutability === is_immutable_type_1.Immutability.ReadonlyShallow ||
                    sourceImmutability === is_immutable_type_1.Immutability.Mutable));
        });
    }),
    defaultOptions: [],
});
exports.default = noUnsafeMutableReadonlyAssignment;
