"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable sonarjs/no-duplicate-string */
const require_strict_mode_1 = __importDefault(require("./require-strict-mode"));
const utils_1 = require("@typescript-eslint/utils");
// eslint-disable-next-line functional/prefer-immutable-types
const ruleTesterForTSConfig = (config) => new utils_1.ESLintUtils.RuleTester({
    parserOptions: {
        sourceType: "module",
        project: config,
        // EXPERIMENTAL_useSourceOfProjectReferenceRedirect: true,
    },
    parser: "@typescript-eslint/parser",
});
// eslint-disable-next-line functional/prefer-immutable-types
const strictRuleTester = ruleTesterForTSConfig("./tsconfig.tests.json");
// eslint-disable-next-line functional/no-expression-statements
strictRuleTester.run("require-strict-mode", require_strict_mode_1.default, {
    valid: [
        {
            filename: "file.ts",
            code: "// strictRuleTester",
        },
    ],
    invalid: [],
});
// eslint-disable-next-line functional/prefer-immutable-types
const nonStrictRuleTester = ruleTesterForTSConfig("./tsconfig.tests.non-strict.json");
// eslint-disable-next-line functional/no-expression-statements
nonStrictRuleTester.run("require-strict-mode", require_strict_mode_1.default, {
    valid: [],
    invalid: [
        {
            filename: "file.ts",
            code: "// nonStrictRuleTester",
            errors: [
                {
                    messageId: "strict",
                    type: utils_1.AST_NODE_TYPES.Program,
                },
            ],
        },
    ],
});
// eslint-disable-next-line functional/prefer-immutable-types
const nonNoUncheckedIndexedAccessRuleTester = ruleTesterForTSConfig("./tsconfig.tests.non-noUncheckedIndexedAccess.json");
// eslint-disable-next-line functional/no-expression-statements
nonNoUncheckedIndexedAccessRuleTester.run("require-strict-mode", require_strict_mode_1.default, {
    valid: [],
    invalid: [
        {
            filename: "file.ts",
            code: "// nonNoUncheckedIndexedAccessRuleTester",
            errors: [
                {
                    messageId: "noUncheckedIndexedAccess",
                    type: utils_1.AST_NODE_TYPES.Program,
                },
            ],
        },
    ],
});
// eslint-disable-next-line functional/prefer-immutable-types
const strictFunctionTypesRuleTester = ruleTesterForTSConfig("./tsconfig.tests.non-strictFunctionTypes.json");
// eslint-disable-next-line functional/no-expression-statements
strictFunctionTypesRuleTester.run("require-strict-mode", require_strict_mode_1.default, {
    valid: [],
    invalid: [
        {
            filename: "file.ts",
            code: "// strictFunctionTypesRuleTester",
            errors: [
                {
                    messageId: "strictFunctionTypes",
                    type: utils_1.AST_NODE_TYPES.Program,
                },
            ],
        },
    ],
});
